SystemOrganization addCategory: #PQTestJsonRPC!

Object subclass: #ArithArg
	instanceVariableNames: 'A B'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PQTestJsonRPC'!

!ArithArg commentStamp: 'PhilippeQuesnel 3/25/2014 01:08' prior: 0!
parameter for Arith jsonrpc server.
methods: Multiply, Divide.

!

!ArithArg methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 23:56'!
a
	^ A! !

!ArithArg methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 23:56'!
a: anObject
	A := anObject! !

!ArithArg methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 23:56'!
b
	^ B! !

!ArithArg methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 23:56'!
b: anObject
	B := anObject! !

Object subclass: #JsonRpcReq
	instanceVariableNames: 'jsonrpc method params id'
	classVariableNames: 'idSeq'
	poolDictionaries: ''
	category: 'PQTestJsonRPC'!

!JsonRpcReq commentStamp: 'PhilippeQuesnel 3/25/2014 01:12' prior: 0!
This is the request object used in JSONRPC calls (v1) to 'call' a method.

id
	- request id 
	
jsonrpc
	- not used w v1 jsonrpc, 
	  v2.0 would use '2.0'

method
	- method name

params
	- array of parameter (eg. ArithArg)
!

JsonRpcReq subclass: #ArithReq
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PQTestJsonRPC'!

!ArithReq commentStamp: 'PhilippeQuesnel 3/25/2014 01:13' prior: 0!
Utility, creates JSON RPC Request object for methods 'Multiply' and 'Divide' of the 'Arith' server.

!

!ArithReq class methodsFor: 'instance creation' stamp: 'PhilippeQuesnel 3/25/2014 00:42'!
divide: arithArg
	^ self method: 'Divide' with: arithArg.
! !

!ArithReq class methodsFor: 'private' stamp: 'PhilippeQuesnel 3/25/2014 00:42'!
method: methodName with: arithArg
	| req |
	req := self new.
	req method: 'Arith.' , methodName.
	req params: { arithArg }. "golang jsonrpc is ver 1, needs to be array"
	^ req.
	! !

!ArithReq class methodsFor: 'instance creation' stamp: 'PhilippeQuesnel 3/25/2014 00:42'!
multiply: arithArg
	^ self method: 'Multiply' with: arithArg.
! !

!JsonRpcReq class methodsFor: 'as yet unclassified' stamp: 'PhilippeQuesnel 3/24/2014 12:35'!
initialize
	super initialize.
	idSeq := 0.! !

!JsonRpcReq class methodsFor: 'as yet unclassified' stamp: 'PhilippeQuesnel 3/24/2014 12:34'!
nextId
	"returns a new Id"
	^ idSeq := idSeq + 1.
! !

!JsonRpcReq methodsFor: 'converting' stamp: 'PhilippeQuesnel 3/24/2014 16:54'!
asDictionary
	^ Dictionary new
		at: #id put: id;
		at: #method put: method;
		at: #params put: params;
		yourself! !

!JsonRpcReq methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 12:29'!
id
	^ id! !

!JsonRpcReq methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 12:29'!
id: anObject
	id := anObject! !

!JsonRpcReq methodsFor: 'initialization' stamp: 'PhilippeQuesnel 3/22/2014 00:58'!
initialize
	super initialize.
	"jsonrpc := '2.0'."	"note: golang jsonrpc is NOT 2.0 !!"
	id := self class nextId.! !

!JsonRpcReq methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 12:30'!
jsonrpc
	^ jsonrpc! !

!JsonRpcReq methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 12:30'!
jsonrpc: anObject
	jsonrpc := anObject! !

!JsonRpcReq methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 12:30'!
method
	^ method! !

!JsonRpcReq methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 12:30'!
method: anObject
	method := anObject! !

!JsonRpcReq methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 12:30'!
params
	^ params! !

!JsonRpcReq methodsFor: 'accessing' stamp: 'PhilippeQuesnel 3/21/2014 12:30'!
params: anObject
	params := anObject! !

!JsonRpcReq methodsFor: 'converting' stamp: 'PhilippeQuesnel 3/24/2014 23:43'!
toJSON
	"convert to JSON string"
	^ String
		streamContents: [ :stream | 
			(NeoJSONWriter on: stream)
				mapInstVarsFor: (self params at: 1) class;
				nextPut: self asDictionary ]! !

Object subclass: #TestJsonRPC
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PQTestJsonRPC'!

!TestJsonRPC commentStamp: 'PhilippeQuesnel 3/25/2014 01:15' prior: 0!
A simple JSON RPC (v1) test, calls Arith.Multiply and Arith.Divide.
Displays on transcript the returned results 'raw': ie the JSON string.
!

!TestJsonRPC methodsFor: 'private' stamp: 'PhilippeQuesnel 3/25/2014 01:03'!
callArith: arithReq
	"call the jsonrpc Arith server, show the result json string"
	| socket resp |

	[ "connect to server"
	socket := Socket newTCP.
	socket connectTo: NetNameResolver localHostAddress port: 8222.
	socket waitForConnectionFor: 1.	
	
	"send request & read response"
	socket sendData: arithReq toJSON.	
	resp := socket receiveData.
	
	] ensure: [socket closeAndDestroy].

	"show response"
	resp ifNotNil: [ Transcript show: arithReq method, ' : ', resp; cr. ].

! !

!TestJsonRPC methodsFor: 'examples' stamp: 'PhilippeQuesnel 3/25/2014 01:04'!
testArith
	"test jsonrpc call Arith.Multiply & Divide"

	| arg arithReq response |
	
	"create ArithArg parameter for Arith.xx"
	arg := ArithArg new.
	arg a: 20.
	arg b: 21.

	"call Arith.Multiply : args"
	arithReq := 	ArithReq multiply: arg.
	response := self callArith: arithReq.

	"call Arith.Divide : args"
	arithReq := 	ArithReq divide: arg.
	response := self callArith: arithReq.
! !
JsonRpcReq initialize!
